@@ -21,7 +21,35 @@
 
 #include "RcppAnnoy.h"
 
-#include "uwot/matrix.h"
+#if ANNOY_VERSION >= Annoy_Version(1,17,3)
+
+typedef Annoy::AnnoyIndexSingleThreadedBuildPolicy AnnoyIndexThreadedBuildPolicy;
+
+struct UwotAnnoyEuclidean {
+  using Distance = Annoy::Euclidean;
+  using S = int32_t;
+  using T = float;
+};
+
+struct UwotAnnoyCosine {
+  using Distance = Annoy::Angular;
+  using S = int32_t;
+  using T = float;
+};
+
+struct UwotAnnoyManhattan {
+  using Distance = Annoy::Manhattan;
+  using S = int32_t;
+  using T = float;
+};
+
+struct UwotAnnoyHamming {
+  using Distance = Annoy::Hamming;
+  using S = int32_t;
+  using T = uint64_t;
+};
+
+#else
 
 typedef AnnoyIndexSingleThreadedBuildPolicy AnnoyIndexThreadedBuildPolicy;
 
@@ -49,6 +77,8 @@ struct UwotAnnoyHamming {
   using T = uint64_t;
 };
 
+#endif  // of 'if ANNOY_VERSION >= Annoy_Version(1,17,3)'
+
 template <typename UwotAnnoyDistance> struct NNWorker {
   const std::string &index_name;
   const std::vector<double> &mat;
@@ -59,10 +89,17 @@ template <typename UwotAnnoyDistance> struct NNWorker {
   std::vector<int> idx;
   std::vector<typename UwotAnnoyDistance::T> dists;
 
+#if ANNOY_VERSION >= Annoy_Version(1,17,3)
+  Annoy::AnnoyIndex<typename UwotAnnoyDistance::S, typename UwotAnnoyDistance::T,
+                    typename UwotAnnoyDistance::Distance, Kiss64Random,
+                    AnnoyIndexThreadedBuildPolicy>
+      index;
+#else
   AnnoyIndex<typename UwotAnnoyDistance::S, typename UwotAnnoyDistance::T,
              typename UwotAnnoyDistance::Distance, Kiss64Random,
              AnnoyIndexThreadedBuildPolicy>
       index;
+#endif
 
   NNWorker(const std::string &index_name, const std::vector<double> &mat,
            std::size_t ncol, std::size_t n_neighbors, std::size_t search_k)
@@ -77,7 +114,9 @@ template <typename UwotAnnoyDistance> struct NNWorker {
   void operator()(std::size_t begin, std::size_t end) {
     for (auto i = begin; i < end; i++) {
       std::vector<typename UwotAnnoyDistance::T> fv(ncol);
-      uwot::get_row(mat, nrow, ncol, i, fv);
+      for (std::size_t j = 0; j < ncol; j++) {
+        fv[j] = mat[i + j * nrow];
+      }
 
       std::vector<typename UwotAnnoyDistance::S> result;
       std::vector<typename UwotAnnoyDistance::T> distances;
@@ -89,8 +128,8 @@ template <typename UwotAnnoyDistance> struct NNWorker {
       }
 
       for (std::size_t j = 0; j < n_neighbors; j++) {
-        uwot::set_row(dists, nrow, n_neighbors, i, distances);
-        uwot::set_row(idx, nrow, n_neighbors, i, result);
+        dists[i + j * nrow] = distances[j];
+        idx[i + j * nrow] = result[j];
       }
     }
   }
